
// (c) 2018 CrownSoft

#ifndef _AR_REMOTE_H_
#define _AR_REMOTE_H_

#include <windows.h>

const wchar_t* const AR_MAIN_WINDOW_CLASS_32 = L"Audio Repeater Pro Class - 32";
const wchar_t* const AR_MAIN_WINDOW_CLASS_64 = L"Audio Repeater Pro Class - 64";
const wchar_t* const AR_MAIN_WINDOW_TEXT = L"Audio Repeater Pro";

namespace ARIPCMessages
{
	enum ARIPCMessages
	{
		LOAD_FX_CHAIN = 6678,
		SET_BYPASS_FX_ENABLE = 6679,
		SET_PROCESSING_ENABLE = 6680,
		SET_STREAM_GAIN = 6681,
		REPEATER_START = 6682,
		REPEATER_STOP = 6683,
		IS_BYPASS_FX_ENABLED = 6684,
		IS_PROCESSING_ENABLED = 6685,
		GET_STREAM_GAIN = 6686,
		IS_REPEATER_STARTED = 6687,
	};
};

class ARRemote
{
protected:
	HWND windowHandle;

public:
	ARRemote()
	{
		windowHandle = NULL;
	}

	bool Connect(bool is32Bit)
	{
		windowHandle = ::FindWindowW(is32Bit ? AR_MAIN_WINDOW_CLASS_32 : AR_MAIN_WINDOW_CLASS_64, AR_MAIN_WINDOW_TEXT);
		return (windowHandle != NULL);
	}

	void LoadFxChain(wchar_t* fileName)
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::LOAD_FX_CHAIN;
		cds.cbData = (DWORD)(sizeof(wchar_t) * (::wcslen(fileName) + 1));
		cds.lpData = fileName;

		::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}

	void SetEnableFxBypass(bool enable)
	{
		int value = (enable ? 1 : 0);

		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::SET_BYPASS_FX_ENABLE;
		cds.cbData = sizeof(int);
		cds.lpData = &value;

		::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}

	void SetEnableProcessing(bool enable)
	{
		int value = (enable ? 1 : 0);

		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::SET_PROCESSING_ENABLE;
		cds.cbData = sizeof(int);
		cds.lpData = &value;

		::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}

	void SetStreamGain(int gain)
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::SET_STREAM_GAIN;
		cds.cbData = sizeof(int);
		cds.lpData = &gain;

		::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}

	void StartRepeater()
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::REPEATER_START;
		cds.cbData = 0;
		cds.lpData = NULL;

		::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}

	void StopRepeater()
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::REPEATER_STOP;
		cds.cbData = 0;
		cds.lpData = NULL;

		::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}

	bool IsBypassFxEnabled()
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::IS_BYPASS_FX_ENABLED;
		cds.cbData = 0;
		cds.lpData = NULL;

		return ((::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds) == 1) ? true : false);
	}

	bool IsProcessingEnabled()
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::IS_PROCESSING_ENABLED;
		cds.cbData = 0;
		cds.lpData = NULL;

		return ((::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds) == 1) ? true : false);
	}

	bool IsRepeaterStarted()
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::IS_REPEATER_STARTED;
		cds.cbData = 0;
		cds.lpData = NULL;

		return ((::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds) == 1) ? true : false);
	}

	int GetStreamGain()
	{
		COPYDATASTRUCT cds;
		cds.dwData = ARIPCMessages::GET_STREAM_GAIN;
		cds.cbData = 0;
		cds.lpData = NULL;

		return (int)::SendMessageW(windowHandle, WM_COPYDATA, (WPARAM)0, (LPARAM)&cds);
	}
};

#endif